@@ -15,6 +15,11 @@
 
 #ifndef _ONEDPL_COMMON_CONFIG_H
 #define _ONEDPL_COMMON_CONFIG_H
+
+#if __cplusplus < 201703L
+#    error "oneDPL requires the C++ language version not less than C++17"
+#endif
+
 // Workarounds for libstdc++9, libstdc++10 when new TBB version is found in the environment
 #if __cplusplus >= 201703L
 #    if __has_include(<tbb/version.h>)
