@@ -1669,7 +1669,7 @@ __pattern_hetero_set_op(_ExecutionPolicy&& __exec, _ForwardIterator1 __first1, _
     auto __buf2 = __keep2(__first2, __last2);
 
     auto __keep3 = oneapi::dpl::__ranges::__get_sycl_range<__par_backend_hetero::access_mode::write, _OutputIterator>();
-    auto __buf3 = __keep3(__result, __result + __n1 + __n2);
+    auto __buf3 = __keep3(__result, __result + __n1);
 
     auto __result_size =
         __par_backend_hetero::__parallel_transform_scan(
