@@ -23,6 +23,8 @@
 
 #include <CL/sycl.hpp>
 
+#include <memory>
+
 // Combine SYCL runtime library version
 #if defined(__LIBSYCL_MAJOR_VERSION) && defined(__LIBSYCL_MINOR_VERSION) && defined(__LIBSYCL_PATCH_VERSION)
 #    define __LIBSYCL_VERSION                                                                                          \
@@ -45,6 +47,8 @@
 #define _ONEDPL_SYCL2020_COLLECTIVES_PRESENT (__LIBSYCL_VERSION >= 50300)
 #define _ONEDPL_SYCL2020_KNOWN_IDENTITY_PRESENT (__LIBSYCL_VERSION >= 50300)
 #define _ONEDPL_SYCL2020_FUNCTIONAL_OBJECTS_PRESENT (__LIBSYCL_VERSION >= 50300)
+#define _ONEDPL_SYCL2023_ATOMIC_REF_PRESENT (__LIBSYCL_VERSION >= 50500)
+#define _ONEDPL_SYCL_SUB_GROUP_MASK_PRESENT (SYCL_EXT_ONEAPI_SUB_GROUP_MASK == 1) && (__LIBSYCL_VERSION >= 50700)
 
 namespace __dpl_sycl
 {
@@ -207,6 +211,29 @@ constexpr __target __target_device =
     __target::global_buffer;
 #endif
 
+template <typename _DataT>
+using __buffer_allocator =
+#if __LIBSYCL_VERSION >= 50707
+    sycl::buffer_allocator<_DataT>;
+#else
+    sycl::buffer_allocator;
+#endif
+
+template <typename _AtomicType, sycl::access::address_space _Space>
+#if _ONEDPL_SYCL2023_ATOMIC_REF_PRESENT
+struct __atomic_ref : sycl::atomic_ref<_AtomicType, sycl::memory_order::relaxed, sycl::memory_scope::work_group, _Space>
+{
+    template <typename _PointerT>
+    explicit __atomic_ref(_PointerT data)
+        : sycl::atomic_ref<_AtomicType, sycl::memory_order::relaxed, sycl::memory_scope::work_group, _Space>(*data){};
+
+    __atomic_ref(const __atomic_ref& other) noexcept
+        : sycl::atomic_ref<_AtomicType, sycl::memory_order::relaxed, sycl::memory_scope::work_group, _Space>(other){};
+};
+#else
+using __atomic_ref = sycl::atomic<_AtomicType, _Space>;
+#endif // _ONEDPL_SYCL2023_ATOMIC_REF_PRESENT
+
 } // namespace __dpl_sycl
 
 #endif /* _ONEDPL_sycl_defs_H */
