@@ -101,7 +101,7 @@ template <sycl::access::mode AccMode = sycl::access::mode::read_write,
 struct all_view_fn
 {
     template <typename _T>
-    _ONEDPL_CONSTEXPR_FUN oneapi::dpl::__ranges::all_view<_T, AccMode, _Target, _Placeholder>
+    constexpr oneapi::dpl::__ranges::all_view<_T, AccMode, _Target, _Placeholder>
     operator()(sycl::buffer<_T, 1> __buf, typename ::std::iterator_traits<_T*>::difference_type __offset = 0,
                typename ::std::iterator_traits<_T*>::difference_type __n = 0) const
     {
@@ -118,17 +118,18 @@ struct all_view_fn
 
 namespace views
 {
-_ONEDPL_CONSTEXPR_VAR
-all_view_fn<sycl::access::mode::read_write, __dpl_sycl::__target_device, sycl::access::placeholder::true_t> all;
+inline constexpr all_view_fn<sycl::access::mode::read_write, __dpl_sycl::__target_device,
+                             sycl::access::placeholder::true_t>
+    all;
 
-_ONEDPL_CONSTEXPR_VAR
-all_view_fn<sycl::access::mode::read, __dpl_sycl::__target_device, sycl::access::placeholder::true_t> all_read;
+inline constexpr all_view_fn<sycl::access::mode::read, __dpl_sycl::__target_device, sycl::access::placeholder::true_t>
+    all_read;
 
-_ONEDPL_CONSTEXPR_VAR
-all_view_fn<sycl::access::mode::write, __dpl_sycl::__target_device, sycl::access::placeholder::true_t> all_write;
+inline constexpr all_view_fn<sycl::access::mode::write, __dpl_sycl::__target_device, sycl::access::placeholder::true_t>
+    all_write;
 
-_ONEDPL_CONSTEXPR_VAR
-all_view_fn<sycl::access::mode::read_write, __dpl_sycl::__target::host_buffer, sycl::access::placeholder::false_t>
+inline constexpr all_view_fn<sycl::access::mode::read_write, __dpl_sycl::__target::host_buffer,
+                             sycl::access::placeholder::false_t>
     host_all;
 } // namespace views
 
@@ -210,7 +211,7 @@ __require_access_zip(sycl::handler& __cgh, oneapi::dpl::__ranges::zip_view<_Rang
 {
     const ::std::size_t __num_ranges = sizeof...(_Ranges);
     oneapi::dpl::__ranges::invoke(__zip.tuple(), _require_access_args<decltype(__cgh)>{__cgh},
-                                  oneapi::dpl::__internal::__make_index_sequence<__num_ranges>());
+                                  ::std::make_index_sequence<__num_ranges>());
 }
 
 //__require_access utility
@@ -259,7 +260,7 @@ template <typename _R>
 struct __range_holder
 {
     _R __r;
-    _ONEDPL_CONSTEXPR_FUN _R
+    constexpr _R
     all_view() const
     {
         return __r;
@@ -296,7 +297,7 @@ struct __buffer_holder
 {
     buf_type<_T> __buf;
 
-    _ONEDPL_CONSTEXPR_FUN oneapi::dpl::__ranges::all_view<_T, AccMode>
+    constexpr oneapi::dpl::__ranges::all_view<_T, AccMode>
     all_view() const
     {
         return oneapi::dpl::__ranges::all_view<_T, AccMode>(__buf);
@@ -350,7 +351,7 @@ struct __get_sycl_range
 
     template <typename _TupleType, typename _DiffType, ::std::size_t... _Ip>
     auto
-    gen_zip_view(_TupleType __t, _DiffType __n, oneapi::dpl::__internal::__index_sequence<_Ip...>)
+    gen_zip_view(_TupleType __t, _DiffType __n, ::std::index_sequence<_Ip...>)
         -> decltype(oneapi::dpl::__ranges::make_zip_view(gen_view(*this, ::std::get<_Ip>(__t), __n).all_view()...))
     {
         auto tmp = oneapi::dpl::__internal::make_tuple(gen_view(*this, ::std::get<_Ip>(__t), __n)...);
@@ -362,17 +363,15 @@ struct __get_sycl_range
 
     template <typename... Iters>
     auto
-    operator()(oneapi::dpl::zip_iterator<Iters...> __first, oneapi::dpl::zip_iterator<Iters...> __last) -> decltype(
-        __range_holder<decltype(gen_zip_view(__first.base(), __last - __first,
-                                             oneapi::dpl::__internal::__make_index_sequence<sizeof...(Iters)>()))>{
-            gen_zip_view(__first.base(), __last - __first,
-                         oneapi::dpl::__internal::__make_index_sequence<sizeof...(Iters)>())})
+    operator()(oneapi::dpl::zip_iterator<Iters...> __first, oneapi::dpl::zip_iterator<Iters...> __last)
+        -> decltype(__range_holder<decltype(gen_zip_view(__first.base(), __last - __first,
+                                                         ::std::make_index_sequence<sizeof...(Iters)>()))>{
+            gen_zip_view(__first.base(), __last - __first, ::std::make_index_sequence<sizeof...(Iters)>())})
     {
         assert(__first < __last);
 
         const ::std::size_t __num_it = sizeof...(Iters);
-        auto rng =
-            gen_zip_view(__first.base(), __last - __first, oneapi::dpl::__internal::__make_index_sequence<__num_it>());
+        auto rng = gen_zip_view(__first.base(), __last - __first, ::std::make_index_sequence<__num_it>());
         return __range_holder<decltype(rng)>{rng};
     }
 
@@ -395,63 +394,38 @@ struct __get_sycl_range
     }
 
   private:
-    //helper SFINAE utilities for a permutation_iterator support
-    template <typename _Map, typename _Size>
-    auto
-    __get_it_map_view(_Map __m, _Size __n) ->
-        typename ::std::enable_if<is_map_iterator<_Map>::value,
-                                  decltype(::std::declval<__get_sycl_range<AccMode, _Iterator>>()(__m,
-                                                                                                  __m + __n))>::type
-    {
-        return this->operator()(__m, __m + __n);
-    }
-    template <typename _Map, typename _Size>
-    auto
-    __get_it_map_view(_Map __m, _Size __n) -> typename ::std::enable_if<is_map_functor<_Map>::value, _Size>::type
-    {
-        return _Size(0);
-    }
-    template <typename _Map, typename _T>
+    template <typename _R, typename _Map, typename _Size,
+              typename ::std::enable_if<oneapi::dpl::__internal::__is_functor<_Map>, int>::type = 0>
     static auto
-    __get_all_view(_Map __m, _T __t) ->
-        typename ::std::enable_if<is_map_iterator<_Map>::value, decltype(__t.all_view())>::type
+    __get_permutation_view(_R __r, _Map __m, _Size __s)
     {
-        return __t.all_view();
+        return oneapi::dpl::__ranges::permutation_view_simple<_R, _Map>{__r, __m, __s};
     }
-    template <typename _Map, typename _T>
-    static auto
-    __get_all_view(_Map __m, _T __t) -> typename ::std::enable_if<is_map_functor<_Map>::value, _Map>::type
+
+    template <
+        typename _R, typename _Map, typename _Size,
+        typename ::std::enable_if<oneapi::dpl::__internal::__is_random_access_iterator<_Map>::value, int>::type = 0>
+    auto
+    __get_permutation_view(_R __r, _Map __m, _Size __s)
     {
-        return __m;
+        auto view_map = this->operator()(__m, __m + __s).all_view();
+        return oneapi::dpl::__ranges::permutation_view_simple<_R, decltype(view_map)>{__r, view_map};
     }
 
   public:
     //specialization for permutation_iterator using sycl_iterator as source
-    template <typename _It, typename _Map>
+    template <typename _It, typename _Map, typename ::std::enable_if<is_hetero_it<_It>::value, int>::type = 0>
     auto
     operator()(oneapi::dpl::permutation_iterator<_It, _Map> __first,
-               oneapi::dpl::permutation_iterator<_It, _Map> __last) ->
-        typename ::std::enable_if<
-            is_hetero_it<_It>::value,
-            __range_holder<oneapi::dpl::__ranges::permutation_view_simple<
-                decltype(::std::declval<__get_sycl_range<AccMode, _Iterator>>()(
-                             __first.base(),
-                             __first.base() + __dpl_sycl::__get_buffer_size(__first.base().get_buffer()))
-                             .all_view()),
-                decltype(__get_all_view(__first.map(), ::std::declval<__get_sycl_range<AccMode, _Iterator>>()
-                                                           .__get_it_map_view(__first.map(), __last - __first)))>>>::
-            type
+               oneapi::dpl::permutation_iterator<_It, _Map> __last)
     {
         auto __n = __last - __first;
         assert(__n > 0);
 
         auto res_src = this->operator()(__first.base(),
                                         __first.base() + __dpl_sycl::__get_buffer_size(__first.base().get_buffer()));
-        auto res_idx = __get_it_map_view(__first.map(), __n);
 
-        auto rng = oneapi::dpl::__ranges::permutation_view_simple<decltype(res_src.all_view()),
-                                                                  decltype(__get_all_view(__first.map(), res_idx))>{
-            res_src.all_view(), __get_all_view(__first.map(), res_idx)};
+        auto rng = __get_permutation_view(res_src.all_view(), __first.map(), __n);
 
         return __range_holder<decltype(rng)>{rng};
     }
@@ -459,18 +433,10 @@ struct __get_sycl_range
     // TODO Add specialization for general case, e.g., permutation_iterator using host
     // or another fancy iterator.
     //specialization for permutation_iterator using USM pointer as source
-    template <typename _It, typename _Map>
+    template <typename _It, typename _Map, typename ::std::enable_if<!is_hetero_it<_It>::value, int>::type = 0>
     auto
     operator()(oneapi::dpl::permutation_iterator<_It, _Map> __first,
-               oneapi::dpl::permutation_iterator<_It, _Map> __last) ->
-        typename ::std::enable_if<
-            !is_hetero_it<_It>::value,
-            __range_holder<oneapi::dpl::__ranges::permutation_view_simple<
-                decltype(
-                    ::std::declval<__get_sycl_range<AccMode, _Iterator>>()(__first.base(), __first.base()).all_view()),
-                decltype(__get_all_view(__first.map(), ::std::declval<__get_sycl_range<AccMode, _Iterator>>()
-                                                           .__get_it_map_view(__first.map(), __last - __first)))>>>::
-            type
+               oneapi::dpl::permutation_iterator<_It, _Map> __last)
     {
         auto __n = __last - __first;
         assert(__n > 0);
@@ -478,11 +444,8 @@ struct __get_sycl_range
         // The size of the source range is unknown. Use non-zero size to create the view.
         // permutation_view_simple access is controlled by the map range view.
         auto res_src = this->operator()(__first.base(), __first.base() + 1 /*source size*/);
-        auto res_idx = __get_it_map_view(__first.map(), __n);
 
-        auto rng = oneapi::dpl::__ranges::permutation_view_simple<decltype(res_src.all_view()),
-                                                                  decltype(__get_all_view(__first.map(), res_idx))>{
-            res_src.all_view(), __get_all_view(__first.map(), res_idx)};
+        auto rng = __get_permutation_view(res_src.all_view(), __first.map(), __n);
 
         return __range_holder<decltype(rng)>{rng};
     }
@@ -525,7 +488,7 @@ struct __get_sycl_range
 
         const auto __offset = __first - oneapi::dpl::begin(__first.get_buffer());
         const auto __n = __last - __first;
-        const auto __size = __first.get_buffer().size();
+        const auto __size = __dpl_sycl::__get_buffer_size(__first.get_buffer());
         assert(__offset + __n <= __size);
 
         return __range_holder<oneapi::dpl::__ranges::all_view<value_type, AccMode>>{
