@@ -68,7 +68,7 @@ struct __make_references
 {
     template <typename _TupleType, ::std::size_t... _Ip>
     _TupleReturnType
-    operator()(const _TupleType& __t, oneapi::dpl::__internal::__index_sequence<_Ip...>)
+    operator()(const _TupleType& __t, ::std::index_sequence<_Ip...>)
     {
         return _TupleReturnType(*::std::get<_Ip>(__t)...);
     }
@@ -101,7 +101,7 @@ class zip_forward_iterator
 
     reference operator*() const
     {
-        return __make_references<reference>()(__my_it_, __make_index_sequence<__num_types>());
+        return __make_references<reference>()(__my_it_, ::std::make_index_sequence<__num_types>());
     }
 
     zip_forward_iterator&
@@ -289,8 +289,8 @@ class zip_iterator
 
     reference operator*() const
     {
-        return oneapi::dpl::__internal::__make_references<reference>()(
-            __my_it_, oneapi::dpl::__internal::__make_index_sequence<__num_types>());
+        return oneapi::dpl::__internal::__make_references<reference>()(__my_it_,
+                                                                       ::std::make_index_sequence<__num_types>());
     }
 
     reference operator[](difference_type __i) const { return *(*this + __i); }
@@ -546,10 +546,34 @@ make_transform_iterator(_Iter __it, _UnaryFunc __unary_func)
     return transform_iterator<_Iter, _UnaryFunc>(__it, __unary_func);
 }
 
-template <typename SourceIterator, typename IndexMap>
+namespace __internal
+{
+
+//functor concept
+struct __functor_concept
+{
+    template <typename _T>
+    static auto
+    test(int) -> decltype(::std::declval<_T>()(0), ::std::true_type{});
+
+    template <typename _T>
+    static auto
+    test(...) -> ::std::false_type;
+};
+
+template <typename _T>
+inline constexpr bool __is_functor = decltype(__functor_concept::test<_T>(0))::value;
+
+} // namespace __internal
+
+template <typename SourceIterator, typename _Permutation>
 class permutation_iterator
 {
   public:
+    typedef typename std::conditional<
+        !__internal::__is_functor<_Permutation>, _Permutation,
+        transform_iterator<counting_iterator<typename ::std::iterator_traits<SourceIterator>::difference_type>,
+                           _Permutation>>::type IndexMap;
     typedef typename ::std::iterator_traits<SourceIterator>::difference_type difference_type;
     typedef typename ::std::iterator_traits<SourceIterator>::value_type value_type;
     typedef typename ::std::iterator_traits<SourceIterator>::pointer pointer;
@@ -560,24 +584,40 @@ class permutation_iterator
 
     permutation_iterator() = default;
 
-    permutation_iterator(const SourceIterator& input1, const IndexMap& input2, ::std::size_t index = 0)
-        : my_source_it(input1), my_index_map(input2), my_index(index)
+    template <typename _T = _Permutation, typename ::std::enable_if<!__internal::__is_functor<_T>, int>::type = 0>
+    permutation_iterator(SourceIterator input1, _Permutation input2) : my_source_it(input1), my_index(input2)
     {
     }
 
+    template <typename _T = _Permutation, typename ::std::enable_if<__internal::__is_functor<_T>, int>::type = 0>
+    permutation_iterator(SourceIterator input1, _Permutation __f, difference_type __idx = 0)
+        : my_source_it(input1), my_index(counting_iterator<difference_type>(__idx), __f)
+    {
+    }
+
+  private:
+    template <typename _T = _Permutation, typename ::std::enable_if<__internal::__is_functor<_T>, int>::type = 0>
+    permutation_iterator(SourceIterator input1, IndexMap input2) : my_source_it(input1), my_index(input2)
+    {
+    }
+
+  public:
     SourceIterator
     base() const
     {
         return my_source_it;
     }
 
-    IndexMap
+    auto
     map() const
     {
-        return my_index_map + my_index;
+        if constexpr (__internal::__is_functor<_Permutation>)
+            return my_index.functor();
+        else
+            return my_index;
     }
 
-    reference operator*() const { return my_source_it[my_index_map[my_index]]; }
+    reference operator*() const { return my_source_it[*my_index]; }
 
     reference operator[](difference_type __i) const { return *(*this + __i); }
 
@@ -614,13 +654,13 @@ class permutation_iterator
     permutation_iterator
     operator+(difference_type forward) const
     {
-        return permutation_iterator(my_source_it, my_index_map, my_index + forward);
+        return permutation_iterator(my_source_it, my_index + forward);
     }
 
     permutation_iterator
     operator-(difference_type backward) const
     {
-        return permutation_iterator(my_source_it, my_index_map, my_index - backward);
+        return permutation_iterator(my_source_it, my_index - backward);
     }
 
     permutation_iterator&
@@ -646,7 +686,7 @@ class permutation_iterator
     friend permutation_iterator
     operator+(difference_type forward, const permutation_iterator& it)
     {
-        return permutation_iterator(it.my_source_it, it.my_index_map, it.my_index + forward);
+        return permutation_iterator(it.my_source_it, it.my_index + forward);
     }
 
     bool
@@ -682,15 +722,14 @@ class permutation_iterator
 
   private:
     SourceIterator my_source_it;
-    IndexMap my_index_map;
-    difference_type my_index;
+    IndexMap my_index;
 };
 
-template <typename SourceIterator, typename IndexMap>
+template <typename SourceIterator, typename IndexMap, typename... StartIndex>
 permutation_iterator<SourceIterator, IndexMap>
-make_permutation_iterator(SourceIterator source, IndexMap map)
+make_permutation_iterator(SourceIterator source, IndexMap map, StartIndex... idx)
 {
-    return permutation_iterator<SourceIterator, IndexMap>(source, map);
+    return permutation_iterator<SourceIterator, IndexMap>(source, map, idx...);
 }
 
 namespace internal
@@ -875,11 +914,9 @@ struct make_zipiterator_functor
 template <typename F, template <typename...> class TBig, typename... T, typename... RestTuples>
 auto
 map_zip(F f, TBig<T...> in, RestTuples... rest)
-    -> decltype(map_tuple_impl(make_zipiterator_functor{},
-                               oneapi::dpl::__internal::__make_index_sequence<sizeof...(T)>(), in, rest...))
+    -> decltype(map_tuple_impl(make_zipiterator_functor{}, ::std::make_index_sequence<sizeof...(T)>(), in, rest...))
 {
-    return map_tuple_impl(make_zipiterator_functor{}, f, oneapi::dpl::__internal::__make_index_sequence<sizeof...(T)>(),
-                          in, rest...);
+    return map_tuple_impl(make_zipiterator_functor{}, f, ::std::make_index_sequence<sizeof...(T)>(), in, rest...);
 }
 
 } // namespace __internal
