@@ -55,20 +55,16 @@ main()
     static_assert(is_execution_policy<unsequenced_policy>::value, "wrong result for is_execution_policy<unsequenced_policy>");
     static_assert(is_execution_policy<parallel_policy>::value, "wrong result for is_execution_policy<parallel_policy>");
     static_assert(is_execution_policy<parallel_unsequenced_policy>::value, "wrong result for is_execution_policy<parallel_unsequenced_policy>");
-#if __cplusplus >= 201402
     static_assert(is_execution_policy_v<sequenced_policy>, "wrong result for is_execution_policy_v<sequenced_policy>");
     static_assert(is_execution_policy_v<unsequenced_policy>, "wrong result for is_execution_policy_v<unsequenced_policy>");
     static_assert(is_execution_policy_v<parallel_policy>, "wrong result for is_execution_policy_v<parallel_policy>");
     static_assert(is_execution_policy_v<parallel_unsequenced_policy>, "wrong result for is_execution_policy_v<parallel_unsequenced_policy>");
-#endif //__cplusplus >= 201402
 
 #if TEST_DPCPP_BACKEND_PRESENT
     auto q = sycl::queue{TestUtils::default_selector};
 
     static_assert(is_execution_policy<device_policy<class Kernel_0>>::value, "wrong result for is_execution_policy<device_policy>");
-#if __cplusplus >= 201402
     static_assert(is_execution_policy_v<device_policy<class Kernel_0>>, "wrong result for is_execution_policy_v<device_policy>");
-#endif //__cplusplus >= 201402
 
     test_policy_instance(dpcpp_default);
 
@@ -92,9 +88,7 @@ main()
 
 #if ONEDPL_FPGA_DEVICE
     static_assert(is_execution_policy<fpga_policy</*unroll_factor =*/ 1, class Kernel_0>>::value, "wrong result for is_execution_policy<fpga_policy>");
-#if __cplusplus >= 201402
     static_assert(is_execution_policy_v<fpga_policy</*unroll_factor =*/ 1, class Kernel_0>>, "wrong result for is_execution_policy_v<fpga_policy>");
-#endif //__cplusplus >= 201402
     test_policy_instance(dpcpp_fpga);
 
     // make_fpga_policy
