@@ -216,11 +216,13 @@ test_shift_by_type(Size m, Size n)
 int
 main()
 {
+    using ValueType = ::std::int32_t;
+
     const ::std::size_t N = 100000;
     for (long m = 0; m < N; m = m < 16 ? m + 1 : long(3.1415 * m))
         for (long n = 0; n < N; n = n < 16 ? n + 1 : long(3.1415 * n))
     {
-       test_shift_by_type<std::int32_t>(m, n);
+       test_shift_by_type<ValueType>(m, n);
     }
 
     return TestUtils::done();
