@@ -80,7 +80,7 @@ main()
 #if TEST_DPCPP_BACKEND_PRESENT
     sycl::queue q;
 #if _ONEDPL_DEBUG_SYCL
-    std::cout << "    Device Name = " << q.get_device().get_info<cl::sycl::info::device::name>().c_str() << "\n";
+    std::cout << "    Device Name = " << q.get_device().get_info<sycl::info::device::name>().c_str() << "\n";
 #endif // _ONEDPL_DEBUG_SYCL
 
     // Run tests for USM shared memory
