@@ -13,6 +13,9 @@
 //
 //===----------------------------------------------------------------------===//
 
+#ifndef __UTILS_H
+#define __UTILS_H
+
 // File contains common utilities that tests rely on
 
 // Do not #include <algorithm>, because if we do we will not detect accidental dependencies.
@@ -693,3 +696,5 @@ struct can_use_default_less_operator<T, decltype(::std::declval<T>() < ::std::de
 };
 
 } /* namespace TestUtils */
+
+#endif // __UTILS_H
